tic;    
[VOI, STATES, ALGEBRAIC, CONSTANTS] = mainFunction();
toc;

function [VOI, STATES, ALGEBRAIC, CONSTANTS] = mainFunction()
    % This is the "main function".  In Matlab, things work best if you rename this function to match the filename.
   [VOI, STATES, ALGEBRAIC, CONSTANTS] = solveModel();
end

function [algebraicVariableCount] = getAlgebraicVariableCount()
    % Used later when setting a global variable with the number of algebraic variables.
    % Note: This is not the "main method".
    algebraicVariableCount =119;
end
% There are a total of 39 entries in each of the rate and state variable arrays.
% There are a total of 106 entries in the constant variable array.
%

function [VOI, STATES, ALGEBRAIC, CONSTANTS] = solveModel()
    [LEGEND_STATES, LEGEND_ALGEBRAIC, LEGEND_VOI, LEGEND_CONSTANTS] = createLegends();
    % Create ALGEBRAIC of correct size
    global algebraicVariableCount;  algebraicVariableCount = getAlgebraicVariableCount();
    % Initialise constants and state variables
    [INIT_STATES, CONSTANTS] = initConsts;

    % Set timespan to solve over
    tspan = [0, 5];

    % Set numerical accuracy options for ODE solver
    options = odeset('RelTol', 1e-06, 'AbsTol', 1e-06, 'MaxStep', 1e-5);

    % Solve model with ODE solver
    [VOI, STATES] = ode15s(@(VOI, STATES)computeRates(VOI, STATES, CONSTANTS), tspan, INIT_STATES, options);

    % Compute algebraic variables
    [RATES, ALGEBRAIC] = computeRates(VOI, STATES, CONSTANTS);
    ALGEBRAIC = computeAlgebraic(ALGEBRAIC, CONSTANTS, STATES, VOI);
    vlabels = {'pH_i in component environment (dimensionless)',...
        'pH_o in component environment (dimensionless)',...
        'q_Cl_i in component environment (fmol)',...
        'q_Na_i in component environment (fmol)',...
        'q_HCO3_i in component environment (fmol)',...
        'q_HB1 in component environment (fmol)'        };
    [~, i_st, i_alg] = find_indices(vlabels, cellstr(LEGEND_CONSTANTS), cellstr(LEGEND_STATES), cellstr(LEGEND_ALGEBRAIC));
    if i_st
        vol_myo = 38; % pL
        plot_selected(i_st,VOI,STATES/vol_myo,'s',LEGEND_STATES,'concentrations mM',ceil(sqrt(length(i_st))))
    end
    plot_selected(i_alg,VOI,ALGEBRAIC,'s',LEGEND_ALGEBRAIC,'ALGEBRAIC',ceil(sqrt(length(i_alg))))

    % fluxes
    vlabels = {
		'v_R12_AE in component AE (fmol_per_sec)',...
		'v_R45_AE in component AE (fmol_per_sec)',...
		'v_R12_CHE in component CHE (fmol_per_sec)',...
		'v_R45_CHE in component CHE (fmol_per_sec)',...
		'v_Re_CO2hyd in component CO2_hyd (fmol_per_sec)',...
		'v_R12_NBC in component NBC (fmol_per_sec)',...
		'v_R45_NBC in component NBC (fmol_per_sec)',...
		'v_R12_NHE in component NHE (fmol_per_sec)',...
		'v_R45_NHE in component NHE (fmol_per_sec)',...
		'v_Re_B1 in component pH_B1 (fmol_per_sec)',...
		'v_Re_B2 in component pH_B2 (fmol_per_sec)'	};
    
	[~, ~, i_alg] = find_indices(vlabels, cellstr(LEGEND_CONSTANTS), cellstr(LEGEND_STATES), cellstr(LEGEND_ALGEBRAIC));
    if true
        plot_selected(i_alg,VOI,ALGEBRAIC,'s',LEGEND_ALGEBRAIC,'FLUXES',ceil(sqrt(length(i_alg))))
    end
    
end            


function [i_con, i_st, i_alg] = find_indices(labels, LEGEND_CONSTANTS, LEGEND_STATES, LEGEND_ALGEBRAIC)
% return the indices for the selected labels
    all_legends = [LEGEND_CONSTANTS; LEGEND_STATES; LEGEND_ALGEBRAIC];
    
    i_con = [];
    for i = 1:length(labels)
        i_con = [i_con; find(strcmp(labels{i},LEGEND_CONSTANTS))];
    end
    i_st = [];
    for i = 1:length(labels)
        i_st = [i_st; find(strcmp(labels{i},LEGEND_STATES))];
    end
    i_alg = [];
    for i = 1:length(labels)
        i_alg = [i_alg; find(strcmp(labels{i},LEGEND_ALGEBRAIC))];
    end
    
    if length(i_con) + length(i_st) + length(i_alg) < length(labels)
        error('missing index');
    end
end


function [] = plot_selected(ids,x,y,legend_x,legend_y,titlestr,ns)
    istart = 1;
%     istart = find(x > 3.2, 1);
    figure();
%     plot stimuli
    for i = 1:length(ids)
        subplot(ns,ns,i)
        plotting_x = x(istart:end);
        plotting_y = y(istart:end,ids(i));
        if isempty(plotting_y) % probably a constant scalar
            plotting_y = repmat(y(ids(i)),2);
            plotting_x = [x(istart), x(end)];
        end
        plot(plotting_x, plotting_y);
        xlabel('time (s)');
        str = split(legend_y(ids(i),:), ' ');
        str = legend_y(ids(i),:);
        l = legend(str);
        set(l,'interpreter','none');
    end
    if verLessThan('matlab','9.5')
        suptitle(titlestr)    
    else
        sgtitle(titlestr)    
    end
end

function [LEGEND_STATES, LEGEND_ALGEBRAIC, LEGEND_VOI, LEGEND_CONSTANTS] = createLegends()
    LEGEND_STATES = ''; LEGEND_ALGEBRAIC = ''; LEGEND_VOI = ''; LEGEND_CONSTANTS = '';
    LEGEND_CONSTANTS(:,1) = strpad('kappa_R61_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,2) = strpad('kappa_R12_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,3) = strpad('kappa_R23_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,4) = strpad('kappa_R34_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,5) = strpad('kappa_R45_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,6) = strpad('kappa_R56_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,7) = strpad('kappa_R61_pHi_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,8) = strpad('kappa_R12_pHi_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,9) = strpad('kappa_R23_pHi_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,10) = strpad('kappa_R34_pHi_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,11) = strpad('kappa_R45_pHi_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,12) = strpad('kappa_R56_pHi_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,13) = strpad('kappa_R61_pHo_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,14) = strpad('kappa_R12_pHo_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,15) = strpad('kappa_R23_pHo_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,16) = strpad('kappa_R34_pHo_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,17) = strpad('kappa_R45_pHo_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,18) = strpad('kappa_R56_pHo_AE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,19) = strpad('kappa_R61_CHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,20) = strpad('kappa_R12_CHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,21) = strpad('kappa_R23_CHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,22) = strpad('kappa_R34_CHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,23) = strpad('kappa_R45_CHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,24) = strpad('kappa_R56_CHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,25) = strpad('kappa_Re_CO2hyd in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,26) = strpad('kappa_R61_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,27) = strpad('kappa_R12_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,28) = strpad('kappa_R23_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,29) = strpad('kappa_R34_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,30) = strpad('kappa_R45_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,31) = strpad('kappa_R56_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,32) = strpad('kappa_R61_pHi_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,33) = strpad('kappa_R12_pHi_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,34) = strpad('kappa_R23_pHi_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,35) = strpad('kappa_R34_pHi_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,36) = strpad('kappa_R45_pHi_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,37) = strpad('kappa_R56_pHi_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,38) = strpad('kappa_R61_pHo_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,39) = strpad('kappa_R12_pHo_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,40) = strpad('kappa_R23_pHo_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,41) = strpad('kappa_R34_pHo_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,42) = strpad('kappa_R45_pHo_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,43) = strpad('kappa_R56_pHo_NBC in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,44) = strpad('kappa_R61_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,45) = strpad('kappa_R12_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,46) = strpad('kappa_R23_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,47) = strpad('kappa_R34_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,48) = strpad('kappa_R45_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,49) = strpad('kappa_R56_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,50) = strpad('kappa_R61_pHi_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,51) = strpad('kappa_R12_pHi_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,52) = strpad('kappa_R23_pHi_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,53) = strpad('kappa_R34_pHi_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,54) = strpad('kappa_R45_pHi_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,55) = strpad('kappa_R56_pHi_NHE in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,56) = strpad('kappa_Re_B1 in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,57) = strpad('kappa_Re_B2 in component BG_parameters (fmol_per_sec)');
    LEGEND_CONSTANTS(:,58) = strpad('K_HCO3_i in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,59) = strpad('K_HCO3_o in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,60) = strpad('K_Cl_i in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,61) = strpad('K_Cl_o in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,62) = strpad('K_S1_AE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,63) = strpad('K_S2_AE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,64) = strpad('K_S3_AE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,65) = strpad('K_S4_AE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,66) = strpad('K_S5_AE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,67) = strpad('K_S6_AE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,68) = strpad('K_H_i in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,69) = strpad('K_H_o in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,70) = strpad('K_OH_i in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,71) = strpad('K_OH_o in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,72) = strpad('K_S1_CHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,73) = strpad('K_S2_CHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,74) = strpad('K_S3_CHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,75) = strpad('K_S4_CHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,76) = strpad('K_S5_CHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,77) = strpad('K_S6_CHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,78) = strpad('K_CO2_i in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,79) = strpad('K_Na_i in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,80) = strpad('K_Na_o in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,81) = strpad('K_S1_NBC in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,82) = strpad('K_S2_NBC in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,83) = strpad('K_S3_NBC in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,84) = strpad('K_S4_NBC in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,85) = strpad('K_S5_NBC in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,86) = strpad('K_S6_NBC in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,87) = strpad('K_S1_NHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,88) = strpad('K_S2_NHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,89) = strpad('K_S3_NHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,90) = strpad('K_S4_NHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,91) = strpad('K_S5_NHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,92) = strpad('K_S6_NHE in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,93) = strpad('K_B1 in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,94) = strpad('K_HB1 in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,95) = strpad('K_B2 in component BG_parameters (per_fmol)');
    LEGEND_CONSTANTS(:,96) = strpad('K_HB2 in component BG_parameters (per_fmol)');
    LEGEND_VOI = strpad('time in component environment (second)');
    LEGEND_STATES(:,1) = strpad('q_HCO3_i in component environment (fmol)');
    LEGEND_STATES(:,2) = strpad('q_HCO3_o in component environment (fmol)');
    LEGEND_STATES(:,3) = strpad('q_Cl_i in component environment (fmol)');
    LEGEND_STATES(:,4) = strpad('q_Cl_o in component environment (fmol)');
    LEGEND_STATES(:,5) = strpad('q_OH_i in component environment (fmol)');
    LEGEND_STATES(:,6) = strpad('q_OH_o in component environment (fmol)');
    LEGEND_STATES(:,7) = strpad('q_Na_i in component environment (fmol)');
    LEGEND_STATES(:,8) = strpad('q_Na_o in component environment (fmol)');
    LEGEND_STATES(:,9) = strpad('q_H_i in component environment (fmol)');
    LEGEND_STATES(:,10) = strpad('q_H_o in component environment (fmol)');
    LEGEND_STATES(:,11) = strpad('q_B1 in component environment (fmol)');
    LEGEND_STATES(:,12) = strpad('q_HB1 in component environment (fmol)');
    LEGEND_STATES(:,13) = strpad('q_B2 in component environment (fmol)');
    LEGEND_STATES(:,14) = strpad('q_HB2 in component environment (fmol)');
    LEGEND_STATES(:,15) = strpad('q_CO2_i in component environment (fmol)');
    LEGEND_ALGEBRAIC(:,17) = strpad('v_R61_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,18) = strpad('v_R12_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,19) = strpad('v_R23_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,20) = strpad('v_R34_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,21) = strpad('v_R45_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,22) = strpad('v_R56_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,24) = strpad('v_R61_pHi_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,26) = strpad('v_R12_pHi_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,28) = strpad('v_R23_pHi_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,30) = strpad('v_R34_pHi_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,33) = strpad('v_R45_pHi_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,35) = strpad('v_R56_pHi_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,39) = strpad('v_R61_pHo_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,41) = strpad('v_R12_pHo_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,43) = strpad('v_R23_pHo_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,45) = strpad('v_R34_pHo_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,47) = strpad('v_R45_pHo_AE in component AE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,49) = strpad('v_R56_pHo_AE in component AE (fmol_per_sec)');
    LEGEND_CONSTANTS(:,97) = strpad('ni in component AE (dimensionless)');
    LEGEND_CONSTANTS(:,98) = strpad('no in component AE (dimensionless)');
    LEGEND_ALGEBRAIC(:,42) = strpad('v_R61_CHE in component CHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,44) = strpad('v_R12_CHE in component CHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,46) = strpad('v_R23_CHE in component CHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,48) = strpad('v_R34_CHE in component CHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,50) = strpad('v_R45_CHE in component CHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,52) = strpad('v_R56_CHE in component CHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,56) = strpad('v_Re_CO2hyd in component CO2_hyd (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,69) = strpad('v_R61_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,70) = strpad('v_R12_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,71) = strpad('v_R23_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,72) = strpad('v_R34_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,73) = strpad('v_R45_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,74) = strpad('v_R56_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,75) = strpad('v_R61_pHi_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,76) = strpad('v_R12_pHi_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,77) = strpad('v_R23_pHi_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,78) = strpad('v_R34_pHi_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,79) = strpad('v_R45_pHi_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,80) = strpad('v_R56_pHi_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,82) = strpad('v_R61_pHo_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,83) = strpad('v_R12_pHo_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,84) = strpad('v_R23_pHo_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,85) = strpad('v_R34_pHo_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,86) = strpad('v_R45_pHo_NBC in component NBC (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,87) = strpad('v_R56_pHo_NBC in component NBC (fmol_per_sec)');
    LEGEND_CONSTANTS(:,99) = strpad('ni in component NBC (dimensionless)');
    LEGEND_CONSTANTS(:,100) = strpad('no in component NBC (dimensionless)');
    LEGEND_ALGEBRAIC(:,99) = strpad('v_R61_NHE in component NHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,100) = strpad('v_R12_NHE in component NHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,101) = strpad('v_R23_NHE in component NHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,102) = strpad('v_R34_NHE in component NHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,103) = strpad('v_R45_NHE in component NHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,104) = strpad('v_R56_NHE in component NHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,105) = strpad('v_R61_pHi_NHE in component NHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,106) = strpad('v_R12_pHi_NHE in component NHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,107) = strpad('v_R23_pHi_NHE in component NHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,108) = strpad('v_R34_pHi_NHE in component NHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,109) = strpad('v_R45_pHi_NHE in component NHE (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,110) = strpad('v_R56_pHi_NHE in component NHE (fmol_per_sec)');
    LEGEND_CONSTANTS(:,101) = strpad('ni in component NHE (dimensionless)');
    LEGEND_ALGEBRAIC(:,115) = strpad('v_Re_B1 in component pH_B1 (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,119) = strpad('v_Re_B2 in component pH_B2 (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,81) = strpad('dH_idt_NBC in component environment (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,111) = strpad('dH_idt_NHE in component environment (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,37) = strpad('dH_idt_AE in component environment (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,88) = strpad('dH_odt_NBC in component environment (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,51) = strpad('dH_odt_AE in component environment (fmol_per_sec)');
    LEGEND_ALGEBRAIC(:,4) = strpad('pH_i in component environment (dimensionless)');
    LEGEND_ALGEBRAIC(:,5) = strpad('pH_o in component environment (dimensionless)');
    LEGEND_ALGEBRAIC(:,1) = strpad('cH_i in component environment (mM)');
    LEGEND_ALGEBRAIC(:,2) = strpad('cH_o in component environment (mM)');
    LEGEND_CONSTANTS(:,102) = strpad('w_i in component environment (pL)');
    LEGEND_CONSTANTS(:,103) = strpad('w_o in component environment (pL)');
    LEGEND_CONSTANTS(:,104) = strpad('R in component constants (J_per_K_per_mol)');
    LEGEND_CONSTANTS(:,105) = strpad('T in component constants (kelvin)');
    LEGEND_CONSTANTS(:,106) = strpad('F in component constants (C_per_mol)');
    LEGEND_STATES(:,16) = strpad('q_S1_AE in component AE (fmol)');
    LEGEND_STATES(:,17) = strpad('q_S2_AE in component AE (fmol)');
    LEGEND_STATES(:,18) = strpad('q_S3_AE in component AE (fmol)');
    LEGEND_STATES(:,19) = strpad('q_S4_AE in component AE (fmol)');
    LEGEND_STATES(:,20) = strpad('q_S5_AE in component AE (fmol)');
    LEGEND_STATES(:,21) = strpad('q_S6_AE in component AE (fmol)');
    LEGEND_ALGEBRAIC(:,3) = strpad('mu_HCO3_i in component AE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,6) = strpad('mu_HCO3_o in component AE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,7) = strpad('mu_Cl_i in component AE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,8) = strpad('mu_Cl_o in component AE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,9) = strpad('mu_S1_AE in component AE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,10) = strpad('mu_S2_AE in component AE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,11) = strpad('mu_S3_AE in component AE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,12) = strpad('mu_S4_AE in component AE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,13) = strpad('mu_S5_AE in component AE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,14) = strpad('mu_S6_AE in component AE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,15) = strpad('mu_H_i in component AE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,16) = strpad('mu_H_o in component AE (J_per_mol)');
    LEGEND_STATES(:,22) = strpad('q_S1_CHE in component CHE (fmol)');
    LEGEND_STATES(:,23) = strpad('q_S2_CHE in component CHE (fmol)');
    LEGEND_STATES(:,24) = strpad('q_S3_CHE in component CHE (fmol)');
    LEGEND_STATES(:,25) = strpad('q_S4_CHE in component CHE (fmol)');
    LEGEND_STATES(:,26) = strpad('q_S5_CHE in component CHE (fmol)');
    LEGEND_STATES(:,27) = strpad('q_S6_CHE in component CHE (fmol)');
    LEGEND_ALGEBRAIC(:,23) = strpad('mu_OH_i in component CHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,25) = strpad('mu_OH_o in component CHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,27) = strpad('mu_Cl_i in component CHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,29) = strpad('mu_Cl_o in component CHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,31) = strpad('mu_S1_CHE in component CHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,32) = strpad('mu_S2_CHE in component CHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,34) = strpad('mu_S3_CHE in component CHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,36) = strpad('mu_S4_CHE in component CHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,38) = strpad('mu_S5_CHE in component CHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,40) = strpad('mu_S6_CHE in component CHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,53) = strpad('mu_CO2_i in component CO2_hyd (J_per_mol)');
    LEGEND_ALGEBRAIC(:,54) = strpad('mu_HCO3_i in component CO2_hyd (J_per_mol)');
    LEGEND_ALGEBRAIC(:,55) = strpad('mu_H_i in component CO2_hyd (J_per_mol)');
    LEGEND_STATES(:,28) = strpad('q_S1_NBC in component NBC (fmol)');
    LEGEND_STATES(:,29) = strpad('q_S2_NBC in component NBC (fmol)');
    LEGEND_STATES(:,30) = strpad('q_S3_NBC in component NBC (fmol)');
    LEGEND_STATES(:,31) = strpad('q_S4_NBC in component NBC (fmol)');
    LEGEND_STATES(:,32) = strpad('q_S5_NBC in component NBC (fmol)');
    LEGEND_STATES(:,33) = strpad('q_S6_NBC in component NBC (fmol)');
    LEGEND_ALGEBRAIC(:,57) = strpad('mu_Na_i in component NBC (J_per_mol)');
    LEGEND_ALGEBRAIC(:,58) = strpad('mu_Na_o in component NBC (J_per_mol)');
    LEGEND_ALGEBRAIC(:,59) = strpad('mu_HCO3_i in component NBC (J_per_mol)');
    LEGEND_ALGEBRAIC(:,60) = strpad('mu_HCO3_o in component NBC (J_per_mol)');
    LEGEND_ALGEBRAIC(:,61) = strpad('mu_S1_NBC in component NBC (J_per_mol)');
    LEGEND_ALGEBRAIC(:,62) = strpad('mu_S2_NBC in component NBC (J_per_mol)');
    LEGEND_ALGEBRAIC(:,63) = strpad('mu_S3_NBC in component NBC (J_per_mol)');
    LEGEND_ALGEBRAIC(:,64) = strpad('mu_S4_NBC in component NBC (J_per_mol)');
    LEGEND_ALGEBRAIC(:,65) = strpad('mu_S5_NBC in component NBC (J_per_mol)');
    LEGEND_ALGEBRAIC(:,66) = strpad('mu_S6_NBC in component NBC (J_per_mol)');
    LEGEND_ALGEBRAIC(:,67) = strpad('mu_H_i in component NBC (J_per_mol)');
    LEGEND_ALGEBRAIC(:,68) = strpad('mu_H_o in component NBC (J_per_mol)');
    LEGEND_STATES(:,34) = strpad('q_S1_NHE in component NHE (fmol)');
    LEGEND_STATES(:,35) = strpad('q_S2_NHE in component NHE (fmol)');
    LEGEND_STATES(:,36) = strpad('q_S3_NHE in component NHE (fmol)');
    LEGEND_STATES(:,37) = strpad('q_S4_NHE in component NHE (fmol)');
    LEGEND_STATES(:,38) = strpad('q_S5_NHE in component NHE (fmol)');
    LEGEND_STATES(:,39) = strpad('q_S6_NHE in component NHE (fmol)');
    LEGEND_ALGEBRAIC(:,89) = strpad('mu_H_i in component NHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,90) = strpad('mu_H_o in component NHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,91) = strpad('mu_Na_i in component NHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,92) = strpad('mu_Na_o in component NHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,93) = strpad('mu_S1_NHE in component NHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,94) = strpad('mu_S2_NHE in component NHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,95) = strpad('mu_S3_NHE in component NHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,96) = strpad('mu_S4_NHE in component NHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,97) = strpad('mu_S5_NHE in component NHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,98) = strpad('mu_S6_NHE in component NHE (J_per_mol)');
    LEGEND_ALGEBRAIC(:,112) = strpad('mu_H_i in component pH_B1 (J_per_mol)');
    LEGEND_ALGEBRAIC(:,113) = strpad('mu_B1 in component pH_B1 (J_per_mol)');
    LEGEND_ALGEBRAIC(:,114) = strpad('mu_HB1 in component pH_B1 (J_per_mol)');
    LEGEND_ALGEBRAIC(:,116) = strpad('mu_H_i in component pH_B2 (J_per_mol)');
    LEGEND_ALGEBRAIC(:,117) = strpad('mu_B2 in component pH_B2 (J_per_mol)');
    LEGEND_ALGEBRAIC(:,118) = strpad('mu_HB2 in component pH_B2 (J_per_mol)');
    LEGEND_RATES(:,1) = strpad('d/dt q_HCO3_i in component environment (fmol)');
    LEGEND_RATES(:,2) = strpad('d/dt q_HCO3_o in component environment (fmol)');
    LEGEND_RATES(:,3) = strpad('d/dt q_Cl_i in component environment (fmol)');
    LEGEND_RATES(:,4) = strpad('d/dt q_Cl_o in component environment (fmol)');
    LEGEND_RATES(:,7) = strpad('d/dt q_Na_i in component environment (fmol)');
    LEGEND_RATES(:,8) = strpad('d/dt q_Na_o in component environment (fmol)');
    LEGEND_RATES(:,5) = strpad('d/dt q_OH_i in component environment (fmol)');
    LEGEND_RATES(:,6) = strpad('d/dt q_OH_o in component environment (fmol)');
    LEGEND_RATES(:,9) = strpad('d/dt q_H_i in component environment (fmol)');
    LEGEND_RATES(:,10) = strpad('d/dt q_H_o in component environment (fmol)');
    LEGEND_RATES(:,11) = strpad('d/dt q_B1 in component environment (fmol)');
    LEGEND_RATES(:,12) = strpad('d/dt q_HB1 in component environment (fmol)');
    LEGEND_RATES(:,13) = strpad('d/dt q_B2 in component environment (fmol)');
    LEGEND_RATES(:,14) = strpad('d/dt q_HB2 in component environment (fmol)');
    LEGEND_RATES(:,15) = strpad('d/dt q_CO2_i in component environment (fmol)');
    LEGEND_RATES(:,16) = strpad('d/dt q_S1_AE in component AE (fmol)');
    LEGEND_RATES(:,17) = strpad('d/dt q_S2_AE in component AE (fmol)');
    LEGEND_RATES(:,18) = strpad('d/dt q_S3_AE in component AE (fmol)');
    LEGEND_RATES(:,19) = strpad('d/dt q_S4_AE in component AE (fmol)');
    LEGEND_RATES(:,20) = strpad('d/dt q_S5_AE in component AE (fmol)');
    LEGEND_RATES(:,21) = strpad('d/dt q_S6_AE in component AE (fmol)');
    LEGEND_RATES(:,22) = strpad('d/dt q_S1_CHE in component CHE (fmol)');
    LEGEND_RATES(:,23) = strpad('d/dt q_S2_CHE in component CHE (fmol)');
    LEGEND_RATES(:,24) = strpad('d/dt q_S3_CHE in component CHE (fmol)');
    LEGEND_RATES(:,25) = strpad('d/dt q_S4_CHE in component CHE (fmol)');
    LEGEND_RATES(:,26) = strpad('d/dt q_S5_CHE in component CHE (fmol)');
    LEGEND_RATES(:,27) = strpad('d/dt q_S6_CHE in component CHE (fmol)');
    LEGEND_RATES(:,28) = strpad('d/dt q_S1_NBC in component NBC (fmol)');
    LEGEND_RATES(:,29) = strpad('d/dt q_S2_NBC in component NBC (fmol)');
    LEGEND_RATES(:,30) = strpad('d/dt q_S3_NBC in component NBC (fmol)');
    LEGEND_RATES(:,31) = strpad('d/dt q_S4_NBC in component NBC (fmol)');
    LEGEND_RATES(:,32) = strpad('d/dt q_S5_NBC in component NBC (fmol)');
    LEGEND_RATES(:,33) = strpad('d/dt q_S6_NBC in component NBC (fmol)');
    LEGEND_RATES(:,34) = strpad('d/dt q_S1_NHE in component NHE (fmol)');
    LEGEND_RATES(:,35) = strpad('d/dt q_S2_NHE in component NHE (fmol)');
    LEGEND_RATES(:,36) = strpad('d/dt q_S3_NHE in component NHE (fmol)');
    LEGEND_RATES(:,37) = strpad('d/dt q_S4_NHE in component NHE (fmol)');
    LEGEND_RATES(:,38) = strpad('d/dt q_S5_NHE in component NHE (fmol)');
    LEGEND_RATES(:,39) = strpad('d/dt q_S6_NHE in component NHE (fmol)');
    LEGEND_STATES  = LEGEND_STATES';
    LEGEND_ALGEBRAIC = LEGEND_ALGEBRAIC';
    LEGEND_RATES = LEGEND_RATES';
    LEGEND_CONSTANTS = LEGEND_CONSTANTS';
end

function [STATES, CONSTANTS] = initConsts()
    VOI = 0; CONSTANTS = []; STATES = []; ALGEBRAIC = [];
    CONSTANTS(:,1) = 16989.7;
    CONSTANTS(:,2) = 9.95603e+09;
    CONSTANTS(:,3) = 3.34396e+12;
    CONSTANTS(:,4) = 124813;
    CONSTANTS(:,5) = 3.53951e+12;
    CONSTANTS(:,6) = 5.65383e+09;
    CONSTANTS(:,7) = 1.51752e-30;
    CONSTANTS(:,8) = 3.89362e-30;
    CONSTANTS(:,9) = 1.01232e-29;
    CONSTANTS(:,10) = 1.09142e-29;
    CONSTANTS(:,11) = 1.17671e-29;
    CONSTANTS(:,12) = 4.25377e-30;
    CONSTANTS(:,13) = 2.47392e+17;
    CONSTANTS(:,14) = 6.34754e+17;
    CONSTANTS(:,15) = 1.65032e+18;
    CONSTANTS(:,16) = 1.77928e+18;
    CONSTANTS(:,17) = 1.91832e+18;
    CONSTANTS(:,18) = 6.93467e+17;
    CONSTANTS(:,19) = 2.66006;
    CONSTANTS(:,20) = 0.00948314;
    CONSTANTS(:,21) = 894374;
    CONSTANTS(:,22) = 0.00338371;
    CONSTANTS(:,23) = 5.20571e+07;
    CONSTANTS(:,24) = 551967;
    CONSTANTS(:,25) = 4.07921e-08;
    CONSTANTS(:,26) = 0.00115162;
    CONSTANTS(:,27) = 1773.83;
    CONSTANTS(:,28) = 0.951701;
    CONSTANTS(:,29) = 0.110974;
    CONSTANTS(:,30) = 0.0103641;
    CONSTANTS(:,31) = 184.891;
    CONSTANTS(:,32) = 7.65077e+19;
    CONSTANTS(:,33) = 1.15584e+20;
    CONSTANTS(:,34) = 6.96298e+20;
    CONSTANTS(:,35) = 2.70285e+20;
    CONSTANTS(:,36) = 1.45878e+19;
    CONSTANTS(:,37) = 2.48755e+19;
    CONSTANTS(:,38) = 1.79055e-16;
    CONSTANTS(:,39) = 2.70508e-16;
    CONSTANTS(:,40) = 1.62959e-15;
    CONSTANTS(:,41) = 6.32563e-16;
    CONSTANTS(:,42) = 3.41407e-17;
    CONSTANTS(:,43) = 5.82174e-17;
    CONSTANTS(:,44) = 2.72703e-10;
    CONSTANTS(:,45) = 8.07856e-12;
    CONSTANTS(:,46) = 0.0305301;
    CONSTANTS(:,47) = 3.45577e-10;
    CONSTANTS(:,48) = 0.00928252;
    CONSTANTS(:,49) = 4.57222e-12;
    CONSTANTS(:,50) = 1.86088e+17;
    CONSTANTS(:,51) = 2.63374e+17;
    CONSTANTS(:,52) = 6.56887e+17;
    CONSTANTS(:,53) = 1.05958e+18;
    CONSTANTS(:,54) = 4.1238e+17;
    CONSTANTS(:,55) = 1.80633e+17;
    CONSTANTS(:,56) = 1.21945e-08;
    CONSTANTS(:,57) = 0;
    CONSTANTS(:,58) = 0.173797;
    CONSTANTS(:,59) = 0.344362;
    CONSTANTS(:,60) = 3.00818e-05;
    CONSTANTS(:,61) = 0.000182919;
    CONSTANTS(:,62) = 0.00536147;
    CONSTANTS(:,63) = 0.000871028;
    CONSTANTS(:,64) = 0.000793158;
    CONSTANTS(:,65) = 0.000749338;
    CONSTANTS(:,66) = 0.000682347;
    CONSTANTS(:,67) = 0.00573416;
    CONSTANTS(:,68) = 3.74048e+06;
    CONSTANTS(:,69) = 1.86796e+07;
    CONSTANTS(:,70) = 0.00283707;
    CONSTANTS(:,71) = 17251.5;
    CONSTANTS(:,72) = 25.3323;
    CONSTANTS(:,73) = 3256.68;
    CONSTANTS(:,74) = 5424.75;
    CONSTANTS(:,75) = 93.2005;
    CONSTANTS(:,76) = 46.3947;
    CONSTANTS(:,77) = 0.434702;
    CONSTANTS(:,78) = 3.17169e+12;
    CONSTANTS(:,79) = 0.000964254;
    CONSTANTS(:,80) = 0.00165864;
    CONSTANTS(:,81) = 4620.34;
    CONSTANTS(:,82) = 2900.41;
    CONSTANTS(:,83) = 127.315;
    CONSTANTS(:,84) = 19248.9;
    CONSTANTS(:,85) = 43706.1;
    CONSTANTS(:,86) = 6619.78;
    CONSTANTS(:,87) = 4.70821e+09;
    CONSTANTS(:,88) = 4.1237e+09;
    CONSTANTS(:,89) = 7.5687e+08;
    CONSTANTS(:,90) = 1.58488e+09;
    CONSTANTS(:,91) = 4.99686e+09;
    CONSTANTS(:,92) = 8.26036e+09;
    CONSTANTS(:,93) = 8.271e-06;
    CONSTANTS(:,94) = 0.000107117;
    CONSTANTS(:,95) = 1e-6;
    CONSTANTS(:,96) = 1e-6;
    STATES(:,1) = 509.2;
    STATES(:,2) = 122.2952;
    STATES(:,3) = 824.6;
    STATES(:,4) = 725.48;
    STATES(:,5) = 5.36764E-06;
    STATES(:,6) = 1.30166E-06;
    STATES(:,7) = 481.46;
    STATES(:,8) = 725.48;
    STATES(:,9) = 2.69019E-06;
    STATES(:,10) = 2.06299E-07;
    STATES(:,11) = 3199.6;
    STATES(:,12) = 1e-12;
    STATES(:,13) = 1117.2;
    STATES(:,14) = 1e-12;
    STATES(:,15) = 46.968;
    CONSTANTS(:,97) = 5.11;
    CONSTANTS(:,98) = 1.44;
    CONSTANTS(:,99) = 2.91;
    CONSTANTS(:,100) = 2.18;
    CONSTANTS(:,101) = 3.18;
    CONSTANTS(:,102) = 38;
    CONSTANTS(:,103) = 5.182;
    CONSTANTS(:,104) = 8.31;
    CONSTANTS(:,105) = 310;
    CONSTANTS(:,106) = 96485;
    STATES(:,16) = 1.666e-13;
    STATES(:,17) = 1.666e-13;
    STATES(:,18) = 1.666e-13;
    STATES(:,19) = 1.666e-13;
    STATES(:,20) = 1.666e-13;
    STATES(:,21) = 1.666e-13;
    STATES(:,22) = 1.666e-13;
    STATES(:,23) = 1.666e-13;
    STATES(:,24) = 1.666e-13;
    STATES(:,25) = 1.666e-13;
    STATES(:,26) = 1.666e-13;
    STATES(:,27) = 1.666e-13;
    STATES(:,28) = 1.666e-13;
    STATES(:,29) = 1.666e-13;
    STATES(:,30) = 1.666e-13;
    STATES(:,31) = 1.666e-13;
    STATES(:,32) = 1.666e-13;
    STATES(:,33) = 1.666e-13;
    STATES(:,34) = 1.666e-13;
    STATES(:,35) = 1.666e-13;
    STATES(:,36) = 1.666e-13;
    STATES(:,37) = 1.666e-13;
    STATES(:,38) = 1.666e-13;
    STATES(:,39) = 1.666e-13;
    if (isempty(STATES)), warning('Initial values for states not set');, end
end

function [RATES, ALGEBRAIC] = computeRates(VOI, STATES, CONSTANTS)
	disp(VOI)
    global algebraicVariableCount;
    statesSize = size(STATES);
    statesColumnCount = statesSize(2);
    if ( statesColumnCount == 1)
        STATES = STATES';
        ALGEBRAIC = zeros(1, algebraicVariableCount);
        utilOnes = 1;
    else
        statesRowCount = statesSize(1);
        ALGEBRAIC = zeros(statesRowCount, algebraicVariableCount);
        RATES = zeros(statesRowCount, statesColumnCount);
        utilOnes = ones(statesRowCount, 1);
    end
    ALGEBRAIC(:,9) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,62).*STATES(:,16));
    ALGEBRAIC(:,14) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,67).*STATES(:,21));
    ALGEBRAIC(:,17) =  CONSTANTS(:,1).*(exp(ALGEBRAIC(:,14)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,9)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,6) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,59).*STATES(:,2));
    ALGEBRAIC(:,10) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,63).*STATES(:,17));
    ALGEBRAIC(:,18) =  CONSTANTS(:,2).*(exp(ALGEBRAIC(:,9)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,10)+ALGEBRAIC(:,6))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,15) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,24) =  CONSTANTS(:,7).*(exp((ALGEBRAIC(:,14)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,9)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,26) =  CONSTANTS(:,8).*(exp((ALGEBRAIC(:,9)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,10)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,16) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,69).*STATES(:,10));
    ALGEBRAIC(:,39) =  CONSTANTS(:,13).*(exp((ALGEBRAIC(:,14) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,9)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,41) =  CONSTANTS(:,14).*(exp((ALGEBRAIC(:,9) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,10)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,16) = ((((ALGEBRAIC(:,17) - ALGEBRAIC(:,18))+ALGEBRAIC(:,24)) - ALGEBRAIC(:,26))+ALGEBRAIC(:,39)) - ALGEBRAIC(:,41);
    ALGEBRAIC(:,8) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,61).*STATES(:,4));
    ALGEBRAIC(:,11) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,64).*STATES(:,18));
    ALGEBRAIC(:,19) =  CONSTANTS(:,3).*(exp((ALGEBRAIC(:,10)+ALGEBRAIC(:,8))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,11)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,28) =  CONSTANTS(:,9).*(exp((ALGEBRAIC(:,10)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,11)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,43) =  CONSTANTS(:,15).*(exp((ALGEBRAIC(:,10) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,11)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,17) = ((((ALGEBRAIC(:,18) - ALGEBRAIC(:,19))+ALGEBRAIC(:,26)) - ALGEBRAIC(:,28))+ALGEBRAIC(:,41)) - ALGEBRAIC(:,43);
    ALGEBRAIC(:,25) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,71).*STATES(:,6));
    ALGEBRAIC(:,31) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,72).*STATES(:,22));
    ALGEBRAIC(:,32) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,73).*STATES(:,23));
    ALGEBRAIC(:,44) =  CONSTANTS(:,20).*(exp(ALGEBRAIC(:,31)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,32)+ALGEBRAIC(:,25))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,6) = ALGEBRAIC(:,44);
    ALGEBRAIC(:,12) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,65).*STATES(:,19));
    ALGEBRAIC(:,20) =  CONSTANTS(:,4).*(exp(ALGEBRAIC(:,11)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,12)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,30) =  CONSTANTS(:,10).*(exp((ALGEBRAIC(:,11)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,12)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,45) =  CONSTANTS(:,16).*(exp((ALGEBRAIC(:,11) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,12)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,18) = ((((ALGEBRAIC(:,19) - ALGEBRAIC(:,20))+ALGEBRAIC(:,28)) - ALGEBRAIC(:,30))+ALGEBRAIC(:,43)) - ALGEBRAIC(:,45);
    ALGEBRAIC(:,40) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,77).*STATES(:,27));
    ALGEBRAIC(:,42) =  CONSTANTS(:,19).*(exp(ALGEBRAIC(:,40)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,31)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,22) = ALGEBRAIC(:,42) - ALGEBRAIC(:,44);
    ALGEBRAIC(:,29) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,61).*STATES(:,4));
    ALGEBRAIC(:,34) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,74).*STATES(:,24));
    ALGEBRAIC(:,46) =  CONSTANTS(:,21).*(exp((ALGEBRAIC(:,32)+ALGEBRAIC(:,29))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,34)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,4) =  - ALGEBRAIC(:,19) - ALGEBRAIC(:,46);
    ALGEBRAIC(:,7) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,60).*STATES(:,3));
    ALGEBRAIC(:,13) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,66).*STATES(:,20));
    ALGEBRAIC(:,21) =  CONSTANTS(:,5).*(exp(ALGEBRAIC(:,12)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,13)+ALGEBRAIC(:,7))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,33) =  CONSTANTS(:,11).*(exp((ALGEBRAIC(:,12)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,13)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,47) =  CONSTANTS(:,17).*(exp((ALGEBRAIC(:,12) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,13)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,19) = ((((ALGEBRAIC(:,20) - ALGEBRAIC(:,21))+ALGEBRAIC(:,30)) - ALGEBRAIC(:,33))+ALGEBRAIC(:,45)) - ALGEBRAIC(:,47);
    RATES(:,23) = ALGEBRAIC(:,44) - ALGEBRAIC(:,46);
    ALGEBRAIC(:,3) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,58).*STATES(:,1));
    ALGEBRAIC(:,22) =  CONSTANTS(:,6).*(exp((ALGEBRAIC(:,13)+ALGEBRAIC(:,3))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,14)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,35) =  CONSTANTS(:,12).*(exp((ALGEBRAIC(:,13)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,14)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,49) =  CONSTANTS(:,18).*(exp((ALGEBRAIC(:,13) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,14)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,20) = ((((ALGEBRAIC(:,21) - ALGEBRAIC(:,22))+ALGEBRAIC(:,33)) - ALGEBRAIC(:,35))+ALGEBRAIC(:,47)) - ALGEBRAIC(:,49);
    RATES(:,21) = ((((ALGEBRAIC(:,22) - ALGEBRAIC(:,17))+ALGEBRAIC(:,35)) - ALGEBRAIC(:,24))+ALGEBRAIC(:,49)) - ALGEBRAIC(:,39);
    ALGEBRAIC(:,36) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,75).*STATES(:,25));
    ALGEBRAIC(:,48) =  CONSTANTS(:,22).*(exp(ALGEBRAIC(:,34)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,36)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,24) = ALGEBRAIC(:,46) - ALGEBRAIC(:,48);
    ALGEBRAIC(:,27) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,60).*STATES(:,3));
    ALGEBRAIC(:,38) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,76).*STATES(:,26));
    ALGEBRAIC(:,50) =  CONSTANTS(:,23).*(exp(ALGEBRAIC(:,36)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,38)+ALGEBRAIC(:,27))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,3) = ALGEBRAIC(:,21)+ALGEBRAIC(:,50);
    RATES(:,25) = ALGEBRAIC(:,48) - ALGEBRAIC(:,50);
    ALGEBRAIC(:,23) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,70).*STATES(:,5));
    ALGEBRAIC(:,52) =  CONSTANTS(:,24).*(exp((ALGEBRAIC(:,38)+ALGEBRAIC(:,23))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,40)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,5) =  - ALGEBRAIC(:,52);
    RATES(:,26) = ALGEBRAIC(:,50) - ALGEBRAIC(:,52);
    RATES(:,27) = ALGEBRAIC(:,52) - ALGEBRAIC(:,42);
    ALGEBRAIC(:,53) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,78).*STATES(:,15));
    ALGEBRAIC(:,54) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,58).*STATES(:,1));
    ALGEBRAIC(:,55) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,56) =  CONSTANTS(:,25).*(exp(ALGEBRAIC(:,53)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,54)+ALGEBRAIC(:,55))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,15) =  - ALGEBRAIC(:,56);
    ALGEBRAIC(:,60) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,59).*STATES(:,2));
    ALGEBRAIC(:,62) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,82).*STATES(:,29));
    ALGEBRAIC(:,63) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,83).*STATES(:,30));
    ALGEBRAIC(:,71) =  CONSTANTS(:,28).*(exp((ALGEBRAIC(:,60)+ALGEBRAIC(:,62))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,63)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,2) = ALGEBRAIC(:,18) - ALGEBRAIC(:,71);
    ALGEBRAIC(:,59) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,58).*STATES(:,1));
    ALGEBRAIC(:,64) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,84).*STATES(:,31));
    ALGEBRAIC(:,65) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,85).*STATES(:,32));
    ALGEBRAIC(:,73) =  CONSTANTS(:,30).*(exp(ALGEBRAIC(:,64)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,65)+ALGEBRAIC(:,59))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,1) =  - ALGEBRAIC(:,22)+ALGEBRAIC(:,73)+ALGEBRAIC(:,56);
    ALGEBRAIC(:,61) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,81).*STATES(:,28));
    ALGEBRAIC(:,66) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,86).*STATES(:,33));
    ALGEBRAIC(:,69) =  CONSTANTS(:,26).*(exp(ALGEBRAIC(:,66)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,61)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,58) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,80).*STATES(:,8));
    ALGEBRAIC(:,70) =  CONSTANTS(:,27).*(exp((ALGEBRAIC(:,58)+ALGEBRAIC(:,61))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,62)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,67) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,75) =  CONSTANTS(:,32).*(exp((ALGEBRAIC(:,66) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,61)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,76) =  CONSTANTS(:,33).*(exp((ALGEBRAIC(:,61) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,62)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,68) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,69).*STATES(:,10));
    ALGEBRAIC(:,82) =  CONSTANTS(:,38).*(exp((ALGEBRAIC(:,66)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,61)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,83) =  CONSTANTS(:,39).*(exp((ALGEBRAIC(:,61)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,62)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,28) = ((((ALGEBRAIC(:,69) - ALGEBRAIC(:,70))+ALGEBRAIC(:,75)) - ALGEBRAIC(:,76))+ALGEBRAIC(:,82)) - ALGEBRAIC(:,83);
    ALGEBRAIC(:,77) =  CONSTANTS(:,34).*(exp((ALGEBRAIC(:,62) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,63)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,84) =  CONSTANTS(:,40).*(exp((ALGEBRAIC(:,62)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,63)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,29) = ((((ALGEBRAIC(:,70) - ALGEBRAIC(:,71))+ALGEBRAIC(:,76)) - ALGEBRAIC(:,77))+ALGEBRAIC(:,83)) - ALGEBRAIC(:,84);
    ALGEBRAIC(:,72) =  CONSTANTS(:,29).*(exp(ALGEBRAIC(:,63)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,64)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,78) =  CONSTANTS(:,35).*(exp((ALGEBRAIC(:,63) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,64)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,85) =  CONSTANTS(:,41).*(exp((ALGEBRAIC(:,63)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,64)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,30) = ((((ALGEBRAIC(:,71) - ALGEBRAIC(:,72))+ALGEBRAIC(:,77)) - ALGEBRAIC(:,78))+ALGEBRAIC(:,84)) - ALGEBRAIC(:,85);
    ALGEBRAIC(:,79) =  CONSTANTS(:,36).*(exp((ALGEBRAIC(:,64) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,65)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,86) =  CONSTANTS(:,42).*(exp((ALGEBRAIC(:,64)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,65)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,31) = ((((ALGEBRAIC(:,72) - ALGEBRAIC(:,73))+ALGEBRAIC(:,78)) - ALGEBRAIC(:,79))+ALGEBRAIC(:,85)) - ALGEBRAIC(:,86);
    ALGEBRAIC(:,57) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,79).*STATES(:,7));
    ALGEBRAIC(:,74) =  CONSTANTS(:,31).*(exp(ALGEBRAIC(:,65)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,66)+ALGEBRAIC(:,57))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,80) =  CONSTANTS(:,37).*(exp((ALGEBRAIC(:,65) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,66)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,87) =  CONSTANTS(:,43).*(exp((ALGEBRAIC(:,65)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,66)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,32) = ((((ALGEBRAIC(:,73) - ALGEBRAIC(:,74))+ALGEBRAIC(:,79)) - ALGEBRAIC(:,80))+ALGEBRAIC(:,86)) - ALGEBRAIC(:,87);
    RATES(:,33) = ((((ALGEBRAIC(:,74) - ALGEBRAIC(:,69))+ALGEBRAIC(:,80)) - ALGEBRAIC(:,75))+ALGEBRAIC(:,87)) - ALGEBRAIC(:,82);
    ALGEBRAIC(:,90) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,69).*STATES(:,10));
    ALGEBRAIC(:,93) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,87).*STATES(:,34));
    ALGEBRAIC(:,94) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,88).*STATES(:,35));
    ALGEBRAIC(:,100) =  CONSTANTS(:,45).*(exp(ALGEBRAIC(:,93)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,94)+ALGEBRAIC(:,90))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,88) =   - CONSTANTS(:,100).*(ALGEBRAIC(:,82)+ALGEBRAIC(:,83)+ALGEBRAIC(:,84)+ALGEBRAIC(:,85)+ALGEBRAIC(:,86)+ALGEBRAIC(:,87));
    ALGEBRAIC(:,51) =  CONSTANTS(:,98).*(ALGEBRAIC(:,39)+ALGEBRAIC(:,41)+ALGEBRAIC(:,43)+ALGEBRAIC(:,45)+ALGEBRAIC(:,47)+ALGEBRAIC(:,49));
    RATES(:,10) = ALGEBRAIC(:,100)+ALGEBRAIC(:,88)+ALGEBRAIC(:,51);
    ALGEBRAIC(:,92) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,80).*STATES(:,8));
    ALGEBRAIC(:,95) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,89).*STATES(:,36));
    ALGEBRAIC(:,101) =  CONSTANTS(:,46).*(exp((ALGEBRAIC(:,94)+ALGEBRAIC(:,92))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,95)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,8) =  - ALGEBRAIC(:,70) - ALGEBRAIC(:,101);
    ALGEBRAIC(:,91) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,79).*STATES(:,7));
    ALGEBRAIC(:,96) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,90).*STATES(:,37));
    ALGEBRAIC(:,97) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,91).*STATES(:,38));
    ALGEBRAIC(:,103) =  CONSTANTS(:,48).*(exp(ALGEBRAIC(:,96)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,97)+ALGEBRAIC(:,91))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,7) = ALGEBRAIC(:,74)+ALGEBRAIC(:,103);
    ALGEBRAIC(:,98) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,92).*STATES(:,39));
    ALGEBRAIC(:,99) =  CONSTANTS(:,44).*(exp(ALGEBRAIC(:,98)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,93)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,89) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,105) =  CONSTANTS(:,50).*(exp((ALGEBRAIC(:,98) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,93)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,106) =  CONSTANTS(:,51).*(exp((ALGEBRAIC(:,93) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,94)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,34) = ((ALGEBRAIC(:,99) - ALGEBRAIC(:,100))+ALGEBRAIC(:,105)) - ALGEBRAIC(:,106);
    ALGEBRAIC(:,107) =  CONSTANTS(:,52).*(exp((ALGEBRAIC(:,94) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,95)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,35) = ((ALGEBRAIC(:,100) - ALGEBRAIC(:,101))+ALGEBRAIC(:,106)) - ALGEBRAIC(:,107);
    ALGEBRAIC(:,102) =  CONSTANTS(:,47).*(exp(ALGEBRAIC(:,95)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,96)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,108) =  CONSTANTS(:,53).*(exp((ALGEBRAIC(:,95) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,96)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,36) = ((ALGEBRAIC(:,101) - ALGEBRAIC(:,102))+ALGEBRAIC(:,107)) - ALGEBRAIC(:,108);
    ALGEBRAIC(:,109) =  CONSTANTS(:,54).*(exp((ALGEBRAIC(:,96) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,97)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,37) = ((ALGEBRAIC(:,102) - ALGEBRAIC(:,103))+ALGEBRAIC(:,108)) - ALGEBRAIC(:,109);
    ALGEBRAIC(:,104) =  CONSTANTS(:,49).*(exp((ALGEBRAIC(:,97)+ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,98)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,110) =  CONSTANTS(:,55).*(exp((ALGEBRAIC(:,97) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,98)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,38) = ((ALGEBRAIC(:,103) - ALGEBRAIC(:,104))+ALGEBRAIC(:,109)) - ALGEBRAIC(:,110);
    RATES(:,39) = ((ALGEBRAIC(:,104) - ALGEBRAIC(:,99))+ALGEBRAIC(:,110)) - ALGEBRAIC(:,105);
    ALGEBRAIC(:,112) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,113) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,93).*STATES(:,11));
    ALGEBRAIC(:,114) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,94).*STATES(:,12));
    ALGEBRAIC(:,115) =  CONSTANTS(:,56).*(exp((ALGEBRAIC(:,112)+ALGEBRAIC(:,113)+ 2.00000.*ALGEBRAIC(:,112))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,114)+ 2.00000.*ALGEBRAIC(:,112))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    RATES(:,11) =  - ALGEBRAIC(:,115);
    RATES(:,12) = ALGEBRAIC(:,115);
    ALGEBRAIC(:,116) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,117) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,95).*STATES(:,13));
    ALGEBRAIC(:,118) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,96).*STATES(:,14));
    ALGEBRAIC(:,119) =  CONSTANTS(:,57).*(exp((ALGEBRAIC(:,116)+ALGEBRAIC(:,117))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,118)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,81) =  CONSTANTS(:,99).*(ALGEBRAIC(:,75)+ALGEBRAIC(:,76)+ALGEBRAIC(:,77)+ALGEBRAIC(:,78)+ALGEBRAIC(:,79)+ALGEBRAIC(:,80));
    ALGEBRAIC(:,111) =  CONSTANTS(:,101).*(ALGEBRAIC(:,105)+ALGEBRAIC(:,106)+ALGEBRAIC(:,107)+ALGEBRAIC(:,108)+ALGEBRAIC(:,109)+ALGEBRAIC(:,110));
    ALGEBRAIC(:,37) =   - CONSTANTS(:,97).*(ALGEBRAIC(:,24)+ALGEBRAIC(:,26)+ALGEBRAIC(:,28)+ALGEBRAIC(:,30)+ALGEBRAIC(:,33)+ALGEBRAIC(:,35));
    RATES(:,9) = (( - ALGEBRAIC(:,104) - ALGEBRAIC(:,115)) - ALGEBRAIC(:,119))+ALGEBRAIC(:,56)+ALGEBRAIC(:,81)+ALGEBRAIC(:,111)+ALGEBRAIC(:,37);
    RATES(:,13) =  - ALGEBRAIC(:,119);
    RATES(:,14) = ALGEBRAIC(:,119);
   RATES = RATES';
end

% Calculate algebraic variables
function ALGEBRAIC = computeAlgebraic(ALGEBRAIC, CONSTANTS, STATES, VOI)
    statesSize = size(STATES);
    statesColumnCount = statesSize(2);
    if ( statesColumnCount == 1)
        STATES = STATES';
        utilOnes = 1;
    else
        statesRowCount = statesSize(1);
        utilOnes = ones(statesRowCount, 1);
    end
    ALGEBRAIC(:,9) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,62).*STATES(:,16));
    ALGEBRAIC(:,14) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,67).*STATES(:,21));
    ALGEBRAIC(:,17) =  CONSTANTS(:,1).*(exp(ALGEBRAIC(:,14)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,9)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,6) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,59).*STATES(:,2));
    ALGEBRAIC(:,10) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,63).*STATES(:,17));
    ALGEBRAIC(:,18) =  CONSTANTS(:,2).*(exp(ALGEBRAIC(:,9)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,10)+ALGEBRAIC(:,6))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,15) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,24) =  CONSTANTS(:,7).*(exp((ALGEBRAIC(:,14)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,9)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,26) =  CONSTANTS(:,8).*(exp((ALGEBRAIC(:,9)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,10)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,16) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,69).*STATES(:,10));
    ALGEBRAIC(:,39) =  CONSTANTS(:,13).*(exp((ALGEBRAIC(:,14) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,9)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,41) =  CONSTANTS(:,14).*(exp((ALGEBRAIC(:,9) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,10)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,8) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,61).*STATES(:,4));
    ALGEBRAIC(:,11) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,64).*STATES(:,18));
    ALGEBRAIC(:,19) =  CONSTANTS(:,3).*(exp((ALGEBRAIC(:,10)+ALGEBRAIC(:,8))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,11)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,28) =  CONSTANTS(:,9).*(exp((ALGEBRAIC(:,10)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,11)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,43) =  CONSTANTS(:,15).*(exp((ALGEBRAIC(:,10) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,11)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,25) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,71).*STATES(:,6));
    ALGEBRAIC(:,31) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,72).*STATES(:,22));
    ALGEBRAIC(:,32) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,73).*STATES(:,23));
    ALGEBRAIC(:,44) =  CONSTANTS(:,20).*(exp(ALGEBRAIC(:,31)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,32)+ALGEBRAIC(:,25))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,12) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,65).*STATES(:,19));
    ALGEBRAIC(:,20) =  CONSTANTS(:,4).*(exp(ALGEBRAIC(:,11)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,12)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,30) =  CONSTANTS(:,10).*(exp((ALGEBRAIC(:,11)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,12)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,45) =  CONSTANTS(:,16).*(exp((ALGEBRAIC(:,11) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,12)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,40) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,77).*STATES(:,27));
    ALGEBRAIC(:,42) =  CONSTANTS(:,19).*(exp(ALGEBRAIC(:,40)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,31)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,29) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,61).*STATES(:,4));
    ALGEBRAIC(:,34) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,74).*STATES(:,24));
    ALGEBRAIC(:,46) =  CONSTANTS(:,21).*(exp((ALGEBRAIC(:,32)+ALGEBRAIC(:,29))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,34)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,7) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,60).*STATES(:,3));
    ALGEBRAIC(:,13) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,66).*STATES(:,20));
    ALGEBRAIC(:,21) =  CONSTANTS(:,5).*(exp(ALGEBRAIC(:,12)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,13)+ALGEBRAIC(:,7))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,33) =  CONSTANTS(:,11).*(exp((ALGEBRAIC(:,12)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,13)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,47) =  CONSTANTS(:,17).*(exp((ALGEBRAIC(:,12) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,13)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,3) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,58).*STATES(:,1));
    ALGEBRAIC(:,22) =  CONSTANTS(:,6).*(exp((ALGEBRAIC(:,13)+ALGEBRAIC(:,3))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,14)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,35) =  CONSTANTS(:,12).*(exp((ALGEBRAIC(:,13)+ CONSTANTS(:,97).*ALGEBRAIC(:,15))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,14)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,49) =  CONSTANTS(:,18).*(exp((ALGEBRAIC(:,13) -  CONSTANTS(:,98).*ALGEBRAIC(:,16))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,14)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,36) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,75).*STATES(:,25));
    ALGEBRAIC(:,48) =  CONSTANTS(:,22).*(exp(ALGEBRAIC(:,34)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,36)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,27) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,60).*STATES(:,3));
    ALGEBRAIC(:,38) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,76).*STATES(:,26));
    ALGEBRAIC(:,50) =  CONSTANTS(:,23).*(exp(ALGEBRAIC(:,36)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,38)+ALGEBRAIC(:,27))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,23) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,70).*STATES(:,5));
    ALGEBRAIC(:,52) =  CONSTANTS(:,24).*(exp((ALGEBRAIC(:,38)+ALGEBRAIC(:,23))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,40)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,53) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,78).*STATES(:,15));
    ALGEBRAIC(:,54) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,58).*STATES(:,1));
    ALGEBRAIC(:,55) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,56) =  CONSTANTS(:,25).*(exp(ALGEBRAIC(:,53)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,54)+ALGEBRAIC(:,55))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,60) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,59).*STATES(:,2));
    ALGEBRAIC(:,62) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,82).*STATES(:,29));
    ALGEBRAIC(:,63) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,83).*STATES(:,30));
    ALGEBRAIC(:,71) =  CONSTANTS(:,28).*(exp((ALGEBRAIC(:,60)+ALGEBRAIC(:,62))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,63)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,59) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,58).*STATES(:,1));
    ALGEBRAIC(:,64) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,84).*STATES(:,31));
    ALGEBRAIC(:,65) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,85).*STATES(:,32));
    ALGEBRAIC(:,73) =  CONSTANTS(:,30).*(exp(ALGEBRAIC(:,64)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,65)+ALGEBRAIC(:,59))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,61) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,81).*STATES(:,28));
    ALGEBRAIC(:,66) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,86).*STATES(:,33));
    ALGEBRAIC(:,69) =  CONSTANTS(:,26).*(exp(ALGEBRAIC(:,66)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,61)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,58) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,80).*STATES(:,8));
    ALGEBRAIC(:,70) =  CONSTANTS(:,27).*(exp((ALGEBRAIC(:,58)+ALGEBRAIC(:,61))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,62)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,67) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,75) =  CONSTANTS(:,32).*(exp((ALGEBRAIC(:,66) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,61)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,76) =  CONSTANTS(:,33).*(exp((ALGEBRAIC(:,61) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,62)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,68) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,69).*STATES(:,10));
    ALGEBRAIC(:,82) =  CONSTANTS(:,38).*(exp((ALGEBRAIC(:,66)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,61)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,83) =  CONSTANTS(:,39).*(exp((ALGEBRAIC(:,61)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,62)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,77) =  CONSTANTS(:,34).*(exp((ALGEBRAIC(:,62) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,63)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,84) =  CONSTANTS(:,40).*(exp((ALGEBRAIC(:,62)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,63)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,72) =  CONSTANTS(:,29).*(exp(ALGEBRAIC(:,63)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,64)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,78) =  CONSTANTS(:,35).*(exp((ALGEBRAIC(:,63) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,64)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,85) =  CONSTANTS(:,41).*(exp((ALGEBRAIC(:,63)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,64)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,79) =  CONSTANTS(:,36).*(exp((ALGEBRAIC(:,64) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,65)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,86) =  CONSTANTS(:,42).*(exp((ALGEBRAIC(:,64)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,65)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,57) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,79).*STATES(:,7));
    ALGEBRAIC(:,74) =  CONSTANTS(:,31).*(exp(ALGEBRAIC(:,65)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,66)+ALGEBRAIC(:,57))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,80) =  CONSTANTS(:,37).*(exp((ALGEBRAIC(:,65) -  CONSTANTS(:,99).*ALGEBRAIC(:,67))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,66)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,87) =  CONSTANTS(:,43).*(exp((ALGEBRAIC(:,65)+ CONSTANTS(:,100).*ALGEBRAIC(:,68))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,66)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,90) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,69).*STATES(:,10));
    ALGEBRAIC(:,93) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,87).*STATES(:,34));
    ALGEBRAIC(:,94) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,88).*STATES(:,35));
    ALGEBRAIC(:,100) =  CONSTANTS(:,45).*(exp(ALGEBRAIC(:,93)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,94)+ALGEBRAIC(:,90))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,88) =   - CONSTANTS(:,100).*(ALGEBRAIC(:,82)+ALGEBRAIC(:,83)+ALGEBRAIC(:,84)+ALGEBRAIC(:,85)+ALGEBRAIC(:,86)+ALGEBRAIC(:,87));
    ALGEBRAIC(:,51) =  CONSTANTS(:,98).*(ALGEBRAIC(:,39)+ALGEBRAIC(:,41)+ALGEBRAIC(:,43)+ALGEBRAIC(:,45)+ALGEBRAIC(:,47)+ALGEBRAIC(:,49));
    ALGEBRAIC(:,92) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,80).*STATES(:,8));
    ALGEBRAIC(:,95) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,89).*STATES(:,36));
    ALGEBRAIC(:,101) =  CONSTANTS(:,46).*(exp((ALGEBRAIC(:,94)+ALGEBRAIC(:,92))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,95)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,91) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,79).*STATES(:,7));
    ALGEBRAIC(:,96) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,90).*STATES(:,37));
    ALGEBRAIC(:,97) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,91).*STATES(:,38));
    ALGEBRAIC(:,103) =  CONSTANTS(:,48).*(exp(ALGEBRAIC(:,96)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,97)+ALGEBRAIC(:,91))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,98) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,92).*STATES(:,39));
    ALGEBRAIC(:,99) =  CONSTANTS(:,44).*(exp(ALGEBRAIC(:,98)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,93)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,89) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,105) =  CONSTANTS(:,50).*(exp((ALGEBRAIC(:,98) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,93)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,106) =  CONSTANTS(:,51).*(exp((ALGEBRAIC(:,93) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,94)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,107) =  CONSTANTS(:,52).*(exp((ALGEBRAIC(:,94) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,95)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,102) =  CONSTANTS(:,47).*(exp(ALGEBRAIC(:,95)./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,96)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,108) =  CONSTANTS(:,53).*(exp((ALGEBRAIC(:,95) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,96)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,109) =  CONSTANTS(:,54).*(exp((ALGEBRAIC(:,96) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,97)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,104) =  CONSTANTS(:,49).*(exp((ALGEBRAIC(:,97)+ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,98)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,110) =  CONSTANTS(:,55).*(exp((ALGEBRAIC(:,97) -  CONSTANTS(:,101).*ALGEBRAIC(:,89))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,98)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,112) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,113) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,93).*STATES(:,11));
    ALGEBRAIC(:,114) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,94).*STATES(:,12));
    ALGEBRAIC(:,115) =  CONSTANTS(:,56).*(exp((ALGEBRAIC(:,112)+ALGEBRAIC(:,113)+ 2.00000.*ALGEBRAIC(:,112))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp((ALGEBRAIC(:,114)+ 2.00000.*ALGEBRAIC(:,112))./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,116) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,68).*STATES(:,9));
    ALGEBRAIC(:,117) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,95).*STATES(:,13));
    ALGEBRAIC(:,118) =  CONSTANTS(:,104).*CONSTANTS(:,105).*log( CONSTANTS(:,96).*STATES(:,14));
    ALGEBRAIC(:,119) =  CONSTANTS(:,57).*(exp((ALGEBRAIC(:,116)+ALGEBRAIC(:,117))./( CONSTANTS(:,104).*CONSTANTS(:,105))) - exp(ALGEBRAIC(:,118)./( CONSTANTS(:,104).*CONSTANTS(:,105))));
    ALGEBRAIC(:,81) =  CONSTANTS(:,99).*(ALGEBRAIC(:,75)+ALGEBRAIC(:,76)+ALGEBRAIC(:,77)+ALGEBRAIC(:,78)+ALGEBRAIC(:,79)+ALGEBRAIC(:,80));
    ALGEBRAIC(:,111) =  CONSTANTS(:,101).*(ALGEBRAIC(:,105)+ALGEBRAIC(:,106)+ALGEBRAIC(:,107)+ALGEBRAIC(:,108)+ALGEBRAIC(:,109)+ALGEBRAIC(:,110));
    ALGEBRAIC(:,37) =   - CONSTANTS(:,97).*(ALGEBRAIC(:,24)+ALGEBRAIC(:,26)+ALGEBRAIC(:,28)+ALGEBRAIC(:,30)+ALGEBRAIC(:,33)+ALGEBRAIC(:,35));
    ALGEBRAIC(:,1) = STATES(:,9)./CONSTANTS(:,102);
    ALGEBRAIC(:,2) = STATES(:,10)./CONSTANTS(:,103);
    ALGEBRAIC(:,4) =  - arbitrary_log(ALGEBRAIC(:,1), 10);
    ALGEBRAIC(:,5) =  - arbitrary_log(ALGEBRAIC(:,2), 10);
end

% Compute a logarithm to any base" +
function x = arbitrary_log(a, base)
    x = log(a) ./ log(base);
end

% Pad out or shorten strings to a set length
function strout = strpad(strin)
    req_length = 160;
    insize = size(strin,2);
    if insize > req_length
        strout = strin(1:req_length);
    else
        strout = [strin, blanks(req_length - insize)];
    end
end

